from ._fuzzy_chinese_match import FuzzyChineseMatch
from ._character_to_stroke import Stroke
from ._character_to_radical import Radical
import logging
import sys
log_console = logging.StreamHandler(sys.stderr)
default_logger = logging.getLogger(__name__)
default_logger.setLevel(logging.WARNING)
default_logger.addHandler(log_console)
# PEP0440 compatible formatted version, see:
# https://www.python.org/dev/peps/pep-0440/
#
# Generic release markers:
#   X.Y
#   X.Y.Z   # For bugfix releases
#
# Admissible pre-release markers:
#   X.YaN   # Alpha release
#   X.YbN   # Beta release
#   X.YrcN  # Release Candidate
#   X.Y     # Final release
#
# Dev branch marker is: 'X.Y.dev' or 'X.Y.devN' where N is an integer.
# 'X.Y.dev0' is the canonical version of 'X.Y.dev'
#
__version__ = "0.1.5"

__all__ = ['FuzzyChineseMatch', 'Stroke', 'Radical']



import pkg_resources
import logging
from ._utils import is_cjk
default_logger = logging.getLogger(__name__)


class Radical(object):
    """
    Translate a chinese character into radicals.

    **Parameters**
    ----------

    *dictionary_filepath* : str, default=None. 

    File path for user provided dictionary. Default dictionary will be 
            used if not specified. 

    A valid dictionary should be a "UTF-8" encoded text file, having two 
            columns separated by space. First column is the character and the 
            second column is its corresponding decomposition with each char 
            stands for each Radical. Note, the decomposition does not have to 
            be radicals, it can be numbers or letters, or any sequence of chars 
            you like).

        An example dictionary:

        -------
        思 田心
        疆 弓土畺
        -------
    """

    _default_dictionary_filepath = pkg_resources.resource_filename(
        __name__, 'dict_chinese_radical.txt')

    def __init__(self, dictionary_filepath=None):
        if dictionary_filepath:
            self._dictionary_filepath = dictionary_filepath
        else:
            self._dictionary_filepath = self._default_dictionary_filepath
        self._read_dictionary()

    def _read_dictionary(self):
        self._dictionary = {}
        data = {}
        default_logger.debug('Reading Radical dictionary ...')
        with open(self._dictionary_filepath, encoding="UTF-8") as f:
            for line in f:
                if (self._dictionary_filepath ==
                        self._default_dictionary_filepath):
                    item_list = line.strip().split('\t')
                    key = item_list[0]
                    value = [i.strip().split() for i in item_list[1:]]
                    self._dictionary[key] = value[0]
                else:
                    item_list = line.strip().split(' ')
                    key = item_list[0]
                    value = item_list[1:]
                    self._dictionary[key] = value

    def get_radical(self, character, placeholder='', raise_error=False):
        """
        Decompose a character into radicals based on dictionary. When a character can not be decomposed, itself will be returned. If it's not chinese, a placeholder is returned.

        **Parameters**
        ----------

        *character* : str

        A chinese character to be decomposed.

        *placeholder* : str, default = ''

        Output to be used when the character is not chinese.

        *raise_error*: boolean, default = False

        If true, raise error if a character can not be decomposed. 
            The default action is to show warnings.

        **Returns**
        ----------

        str, decomposition results.
        """
        if raise_error:
            if not is_cjk(character):
                raise Exception(f'Character \'{character}\' is not Chinese.')
            if character in self._dictionary:
                return ''.join(self._dictionary[character])
            else:
                raise Exception(
                    f'Unable to decompose character \'{character}\'.')
        else:
            if not is_cjk(character):
                default_logger.warning(
                    f'Character \'{character}\' is not Chinese.')
                return placeholder
            if character in self._dictionary:
                return ''.join(self._dictionary[character])
            else:
                default_logger.warning(
                    f'Unable to decompose character \'{character}\'.')
                return character


if __name__ == "__main__":
    radical = Radical()
    print("中", radical.get_radical("中"))
    print("王", radical.get_radical("王"))
    print("像", radical.get_radical("像"))



# Credits to this answer https://stackoverflow.com/a/52837006/8673150
# list of cjk codepoint ranges
# tuples indicate the bottom and top of the range, inclusive
cjk_ranges = [(0x4E00, 0x62FF), (0x6300, 0x77FF), (0x7800, 0x8CFF),
              (0x8D00, 0x9FCC), (0x3400, 0x4DB5), (0x20000, 0x215FF),
              (0x21600, 0x230FF), (0x23100, 0x245FF), (0x24600, 0x260FF),
              (0x26100, 0x275FF), (0x27600, 0x290FF), (0x29100, 0x2A6DF),
              (0x2A700, 0x2B734), (0x2B740, 0x2B81D), (0x2B820, 0x2CEAF),
              (0x2CEB0, 0x2EBEF), (0x2F800, 0x2FA1F)]


def is_cjk(char):
    char = ord(char)
    for bottom, top in cjk_ranges:
        if char >= bottom and char <= top:
            return True
    return False